/* 
-------------------------------------------------------------------------
./logicaland 0.1 - a participative global simulation
-------------------------------------------------------------------------
Copyright (c) 2002 ./logicaland (http://www.logicaland.net)
Michael Aschauer, Josef Deinhofer (io@logicland.net)
-------------------------------------------------------------------------
For information on usage and redistribution, and for a DISCLAIMER OF ALL
WARRANTIES, see the file, "LICENSE.txt," in this distribution.  
--------------------------------------------------------
 */

package logica.utility;

import java.io.*;
import java.util.*;
import java.util.zip.*;



public class CSVReader 
{

	private StringTokenizer fieldTokenizer;
	private StringTokenizer lineTokenizer;
	
	private String delimiter;
	private String comment;
	
	private StringBuffer dataBuffer;
	private String 	     lineBuffer;
	private String[][]   dataArray;
	
	private int lineNr;
	private int fieldNr;

	private int currentRow;
	private int currentField;




	public CSVReader(java.io.InputStream in) throws IOException
	{
		this(in,";");
	}	
		
	
	
	
	public CSVReader(java.io.InputStream in, String delimiter) throws IOException
	{	
		int i;
		int c;		
		
		dataBuffer = new StringBuffer();
		
		// READ in Data
		try {
			while( (c = in.read()) > 0)
			{
			
				dataBuffer.append((char)c);
			}
			
		}
		catch (	IOException e)
		{

			throw(e);
		}
		
		this.delimiter = delimiter;
		
		this.lineNr = getLineNumber();		
		this.fieldNr = getFieldNumber();
		
		currentRow = 0;

		dataArray = getArray();
                             
		// set buffer to null; 
		dataBuffer = null;
		lineBuffer = null;
	}
	
	
	
	
	// --------------------------------
	// Public Methods:
	// --------------------------------
					
	
	
        /** return the row count of csv
         */        
	public int countRows()
	{
		return lineNr;
	}
	
	
	
        /** return the row count of csv
         */        
	public int countFields()
	{
		return fieldNr;
	}
		
	
	
	public boolean hasNextRow()
	{
		return (currentRow < lineNr);
	}
        
	public void rewind()
	{
		currentRow = 0;
	}        
	
	
        /** fetches a Row
         * be aware: returns a NULL-pointer if there is no row
         */        
	public String[] fetchRow()
	{		
		if (currentRow <= lineNr) 
		{
			currentRow++;
			return dataArray[currentRow - 1];
		}
		else  					// ??????						
		{
			return null;			// !!!Achtung returned Nullpointer 
		}
	}
	

        /** fetches a row (direct access)
         * be aware: returns a NULL-pointer if there is no row
         * @param requestLine row number to fetch
         *
         */        
	public String[] fetchRow(int requestLine)
	{		
		if (requestLine <= lineNr) 
		{
			return dataArray[requestLine];
		}
		else  					// ??????
		{
			return null;			// !!! Achtung returned Nullpointer 
		}
	}			
	
	public String[] fetchColumn(int requestCol)
	{		
		if (requestCol <= fieldNr) 
		{
			String[] outArray = new String[lineNr];
			for (int i = 0; i < lineNr; i++)
			{
				outArray[i] = dataArray[i][requestCol];
			}
			return outArray;
		}
		else  					// ??????
		{
			return null;			// !!! Achtung returned Nullpointer 
		}
	}			
	
        /** fetches the whole csv as 2d-array
         */        
	public String[][] fetchAll()	
	{
		return dataArray;
	}
	
	
	
	// --------------------------------
	// Private Methods:
	// --------------------------------


	private String[][] getArray()
	{
		String[][] out = new String[lineNr][fieldNr];					
		lineTokenizer = new StringTokenizer(dataBuffer.toString(),"\n");

		for (int i=0; i < lineNr; i++)
		{
			lineBuffer = lineTokenizer.nextToken();		
			fieldTokenizer = new StringTokenizer(lineBuffer,delimiter);
			
			for (int j = 0; j < fieldNr; j++)
			{
				try {
					out[i][j] = fieldTokenizer.nextToken();
				}
				catch ( NoSuchElementException  e )
				{
					out[i][j] = "0";			// Field Missing
				}
			}
		}
			
		return out;	
	}	
	
	
	private int getLineNumber()
	{		
		int count=0;
		
		lineTokenizer = new StringTokenizer(dataBuffer.toString(),"\n");
		count = lineTokenizer.countTokens();
		return count;		
	}
	
	
	
	private int getFieldNumber() 
	{		
		int count = 0;
	
		lineTokenizer = new StringTokenizer(dataBuffer.toString(),"\n");
		
		try {
			lineBuffer = lineTokenizer.nextToken();
			fieldTokenizer = new StringTokenizer(lineBuffer,delimiter);
			count = fieldTokenizer.countTokens();			
			
		}
		catch (NoSuchElementException e)
		{  }
		

					
		return count;	
				
	}					
}	
